'===========================================================================
' Subject: FMS v 2 alpha 8 field editor      Date: October 1, 2004      
' Author: Gary Gunnerson                    Code: Rapid-Q      
' Origin: GaryG51384@aol.com                 Web: http://gunnerson.homestead.com
'===========================================================================
'
' This is a utility program for Flight Model Simulator (FMS) located at 
' http://n.ethz.ch/student/mmoeller/fms/index_e.html. The program exposes
' a small sub-set of the parameters in the .par file to allow easy field
' condition adjustments of control throws, battery, model weight etc.
'
' This program takes as input an FMS 2.8 .par file which contains specific 
' sections describing general model parameters, electric and combustion 
' motor and prop parameters, wing location and size, control surface size, 
' location and deflection, model drag parameters, POLAR lift, drag and moment
' tables for wing surfaces, wheel locations and V-tail mixing parameters.
'
' The program then exposes and edits the field adjustable RC model parameters 
' that all who fly RC are familiar with. We should be able to adjust
' control throws, CG, motor and prop parms etc. without corrupting the other
' information within the author's original .par file.
'
' This program also displays the DirectX 3D .x model file if it finds it in
' the current directory.
' 
' This file is FreeWare. Use at your own risk.
'
$INCLUDE "RAPIDQ.INC"
$ESCAPECHARS ON
$OPTION ICON "mqo-browser.ico"
'$TYPECHECK ON

'-- Light Types for Direct3D viewer
CONST D3DRMLIGHT_AMBIENT = 0
CONST D3DRMLIGHT_POINT = 1
CONST D3DRMLIGHT_SPOT = 2
CONST D3DRMLIGHT_DIRECTIONAL = 3
CONST D3DRMLIGHT_PARALLELPOINT = 4

'-- Renderer modes for Direct3D viewer
CONST D3DRMRENDERMODE_BLENDEDTRANSPARENCY = 1
CONST D3DRMRENDERMODE_SORTEDTRANSPARENCY = 2
CONST D3DRMRENDERMODE_LIGHTINMODELSPACE = 8
CONST D3DRMRENDERMODE_VIEWDEPENDENTSPECULAR = 16
CONST D3DRMRENDERMODE_DISABLESORTEDALPHAZWRITE = 32

'-- Wrap Types for Direct3D viewer
CONST D3DRMWRAP_FLAT = 0
CONST D3DRMWRAP_CYLINDER = 1
CONST D3DRMWRAP_SPHERE = 2
CONST D3DRMWRAP_CHROME = 3
CONST D3DRMWRAP_SHEET = 4
CONST D3DRMWRAP_BOX = 5

'Forward declarations of subroutines required by CREATE
DECLARE SUB FormResize (Sender AS QFORM)
DECLARE SUB OpenPARdialog (Sender AS QMENUITEM)
DECLARE SUB SavePARdialog (Sender AS QMENUITEM)
DECLARE SUB DisplayAbout (Sender as QMENUITEM)
DECLARE SUB DisplayInstruct (Sender as QMENUITEM)
'Direct3D model viewer SUBs
DECLARE SUB dxInitialize(sender AS QDXSCREEN)
DECLARE SUB dxInitializeSurface(sender AS QDXSCREEN)
DECLARE SUB dxTimerExpired
DECLARE SUB upButtonMouseDown(button%, x%, y%, shift%, sender AS QBUTTON)
DECLARE SUB upButtonMouseUp(button%, x%, y%, shift%, sender AS QBUTTON)

CREATE MainForm AS QFORM
    Caption = "FMS 2 alpha 8 plane field editor"
    Width = 760
    Height = 540
    Center
    CREATE MainMenu as QMAINMENU
      CREATE Item1 as QMENUITEM
        Caption = "&File"
        CREATE Item1Sub1 as QMENUITEM
               Caption = "&Open PAR"
               OnClick = OpenPARdialog
        END CREATE
        CREATE Item1Sub2 as QMENUITEM
               Caption = "&Save PAR"
               OnClick = SavePARdialog
        END CREATE
      END CREATE 'Item1
      CREATE Item2 as QMENUITEM
        Caption = "&Help"
        CREATE Item2Sub1 AS QMENUITEM
               Caption = "&About"
               OnClick = DisplayAbout
        END CREATE
        CREATE Item2Sub2 AS QMENUITEM
               Caption = "&Instructions"
               OnClick = DisplayInstruct
        END CREATE
      END CREATE 'Item2
    END CREATE    
    CREATE Label1 AS QLABEL
        Caption = "Model Name"
        Left = 10
        Top = 12
        Width = 64
        Transparent = 1
    END CREATE 'Label1
    CREATE StringGrid1 AS QSTRINGGRID
        AddOptions(goEditing)
        Left = 10
        Top = 36
        Height = 260
        Width = 612
        TabOrder = 1
        EditorMode = 1
        GridLineWidth = 2
        ColCount = 6
        RowCount = 2 'Start with 2 rows
        Separator = ","
        FixedCols = 4
        FixedRows = 1
        Cell(0,0) = "PAR Section"
        ColWidths(0) = 82
        Cell(1,0) = "Airplane Part"
        ColWidths(1) = 96
        Cell(2,0) = "Line#"
        ColWidths(2) = 36
        Cell(3,0) = "Old Value"
        ColWidths(3) = 64
        Cell(4,0) = "New Value
        ColWidths(4) = 64
        Cell(5,0) = "Comment"
        ColWidths(5) = 480
    END CREATE 'StringGrid1
    CREATE ListBox1 AS QLISTBOX 
        Left = 280
        Top = 316
        Width = 320
        Height = 110
        TabOrder = 2
    END CREATE 'ListBox1
    CREATE StatusBar AS QSTATUSBAR
    END CREATE
    CREATE dxScreen AS QDXSCREEN
           Top = 362
           Left = 10
           Width = 200
           Height = 108
           bitCount = 16
           use3D = 1
           useHardware = 1
           onInitialize = dxInitialize
           onInitializeSurface = dxInitializeSurface
    END CREATE 'dxScreen
    CREATE upButton AS QBUTTON
            caption = "--"
            hint = "up"
            width = 30
            left = 220
            top = 390
            tabOrder = 0
            onMouseDown = upButtonMouseDown
            onMouseUp = upButtonMouseUp
    END CREATE
    CREATE downButton AS QBUTTON
            caption = "+"
            hint = "down"
            left = 220
            width = 30
            top = 420
            tabOrder = 0
            onMouseDown = upButtonMouseDown
            onMouseUp = upButtonMouseUp
    END CREATE
    OnResize = FormResize
END CREATE

'Insert your initialization code here

'PAR file stream
DIM parfile as QFILESTREAM
DIM parbuffer as QMEMORYSTREAM
'File Dialogs
DIM OpenDialog1 as QOPENDIALOG
DIM SaveDialog1 as QSAVEDIALOG
'Glogal Variables
DIM linenum# as LONG
DIM gridrow# as LONG
DIM parmcount# as LONG
DIM sectioncount# as LONG
DIM elementcount# as LONG
DIM parfilename$ as STRING
DIM inline$ as STRING
DIM parnames$ as STRING
DIM parsection$ as STRING
DIM subsection$ as STRING
DIM parmselect$ as STRING

'DIM statements for Direct3D model viewer
DIM dxTimer AS QDXTIMER
    dxTimer.enabled = 1
    dxTimer.interval = 0
    dxTimer.activeOnly = 0
    dxTimer.onTimer = dxTimerExpired
DIM frameView AS DOUBLE
    frameView = 4
DIM wrapType AS LONG
    wrapType = D3DRMWRAP_SPHERE
DIM T! AS SINGLE
DIM meshFrame AS QD3DFRAME
DIM meshBuilder AS QD3DMESHBUILDER
DIM wrap AS QD3DWRAP

'--------- Subroutines ---------

'Subroutines for Direct3D Model Viewer

SUB dxInitialize(sender AS QDXSCREEN)
   DIM light AS QD3DLIGHT, ambient AS QD3DLIGHT
   DIM lightFrame AS QD3DFRAME
   dxScreen.createFrame(lightFrame)
   dxScreen.createFrame(meshFrame)
   dxScreen.createLightRGB(D3DRMLIGHT_DIRECTIONAL, 0.9, 0.9, 0.9, light)
   lightFrame.addLight(Light)
   dxScreen.createLightRGB(D3DRMLIGHT_AMBIENT, 0.1, 0.1, 0.1, ambient)
   dxScreen.addLight(ambient)
   meshFrame.setPosition(0, 0, frameView)
   dxScreen.setCameraPosition(0, 0, 0)
END SUB

SUB dxInitializeSurface(sender AS QDXSCREEN)
   dxScreen.setRenderMode(D3DRMRENDERMODE_BLENDEDTRANSPARENCY OR D3DRMRENDERMODE_SORTEDTRANSPARENCY)
END SUB

SUB openxFile (parfilename$)
    xfilename$ = LEFT$(parfilename$,(LEN(parfilename$)-4)) + ".x"
    '-- Remove previous object from frame
    meshFrame.deleteVisual(meshBuilder)
        '-- Rebuilds mesh, removing the old object
    dxScreen.createMeshBuilder(meshBuilder)
    '-- Load a new object
    meshBuilder.load(xfilename$)
    '-- Add this new object to the frame
    meshFrame.addVisual(meshBuilder)
    '-- Set rotation angles
    meshFrame.setRotation(.45,.90,0,.02)
    frameView = 4            
    meshFrame.setPosition(0, 0, frameView)
    doEvents
    dxTimerExpired
END SUB 'openxFile

SUB upButtonMouseDown(button%, x%, y%, shift%, sender AS QBUTTON)
    DEFSNG T!

    T! = TIMER
    IF button% = 0 THEN
        sender.tabOrder = 0
        DO
            IF sender.hint = "up" THEN
                INC(frameView)
                ListBox1.AddItems "frameView = " + STR$(frameView)
                SLEEP .1 'added to slow down zoom speed
            ELSE
                DEC(frameView)
                ListBox1.AddItems "frameView = " + STR$(frameView)
                SLEEP .1
            END IF
            meshFrame.setPosition(0, 0, frameView)
            doEvents
            dxTimerExpired
        LOOP UNTIL sender.tabOrder = 1
    END IF
END SUB

SUB upButtonMouseUp(button%, x%, y%, shift%, sender AS QBUTTON)
    IF button% = 0 THEN
        sender.tabOrder = 1
    END IF
END SUB

SUB dxTimerExpired
   dxScreen.forceUpdate(0,0,50,40)    '-- Update FPS text only
   meshFrame.Move(1)                   '-- This does the animation
   dxScreen.render
   dxScreen.textOut(10,10,"FPS: "+str$(dxTimer.frameRate), &HFFFFFF, -1)
   dxScreen.flip
END SUB

'Subroutines for PAR file editor

SUB FormResize
  'Top of Form objects
  'StringGrid object
  StringGrid1.Height = MainForm.Height * .46
  StringGrid1.Width = MainForm.Width - 28
  'dxScreen object
  dxScreen.Top = StringGrid1.Height + 54
  dxScreen.Width = MainForm.Width * .40
  dxScreen.Height = MainForm.Height - StringGrid1.Height - 130
  'dxScreen buttons
  upButton.Top = dxScreen.Top + (.5 * dxScreen.Height) - 30
  downButton.Top = upButton.Top + 30
  upButton.Left = dxScreen.Width + 20
  downButton.Left = upButton.Left
  'ListBox1 object
  ListBox1.Top = dxScreen.Top
  ListBox1.Left = upButton.Left + 40
  ListBox1.Width = MainForm.Width - ListBox1.Left - 18
  ListBox1.Height = dxScreen.Height
END SUB 'FormResize

SUB DisplayAbout
    SHOWMESSAGE "FMS 2 alpha 8 .par file editor displays only field changeable _
    details of a specific FMS radio control model and then saves the modified _
    parameters to the .par file without modifying any wing, motor or wheel locations _
    Gunnerson c2004 http://gunnerson.homestead.com/files/fms_models.htm"
END SUB 'DisplayAbout

SUB DisplayInstruct
    SHOWMESSAGE "1) Load a .PAR file first 2) edit specific parameters _
    3) Save the modified .PAR file Questions? Email GaryG51384@aol.com"
END SUB 'DisplayInstruct

SUB DisplayWrongParFile
    SHOWMESSAGE "This version of the FMS .par file editor only supports _
    airplane files for FMS 2 alpha 8 at this time. Helicopter files and _
    FMS 2 beta 7 files are not editable with this program. -- Gary"
END SUB 'DisplayWrongParFile

SUB InitGrid
    FOR I = 2 TO StringGrid1.RowCount
        StringGrid1.DeleteRow(2)
    NEXT I
END SUB 'InitGrid

SUB StripLeadingTabsSpaces
    IF LEN(inline$) > 1 AND LEFT$(inline$,1) = CHR$(9) THEN 'check for leading tab characters
      DO
        inline$ = MID$(inline$,2,LEN(inline$)) 'remove tabs padding comment lines
      LOOP UNTIL LEFT$(inline$,1) = "/" OR LEN(inline$) = 1
    END IF
    IF LEN(inline$) > 1 AND LEFT$(inline$,1) = CHR$(32) THEN 'check for leading space characters
      DO
        inline$ = MID$(inline$,2,LEN(inline$)) 'remove tabs padding comment lines
      LOOP UNTIL LEN(inline$) = 1
    END IF      
END SUB 'StripLeadingTabsSpaces

SUB ReadNextLine
    linenum# = linenum# + 1
    inline$ = parbuffer.READLINE()
    StripLeadingTabsSpaces
    IF LEFT$(inline$,2) = "//" THEN 'ignore single comment lines
       linenum# = linenum# + 1
       inline$ = parbuffer.READLINE()
       StripLeadingTabsSpaces
    END IF
    'ListBox1.AddItems STR$(linenum#) + "|" + STR$(LEN(inline$)) + " " + inline$
END SUB 'ReadNextLine    

SUB SkipBlankLines 'Skips blank lines
    DO
      ReadNextLine
    LOOP UNTIL LEN(inline$) > 1
END SUB 'SkipBlankLines

SUB Seek (parsection$) 'Skips forward to a specific .par section
    DO
      ReadNextLine
    LOOP UNTIL INSTR(inline$, parsection$) > 0
    ListBox1.AddItems "Line: " + STR$(linenum#) + " Process " + parsection$ + " section"
END SUB 'Seek

SUB GridParms (parmcount#, parmnames$, parmselect$) 'populates editable StringGrid with .par info
    FOR I = 1 to parmcount#
      selected$ = MID$(parmselect$, I, 1)
      IF selected$ = "1" THEN
         StringGrid1.InsertRow (gridrow#)
         StringGrid1.Cell(0,gridrow#) = parsection$ + STR$(sectioncount#) + subsection$
         StringGrid1.Cell(1,gridrow#) = FIELD$(parmnames$," ",I)
         StringGrid1.Cell(2,gridrow#) = STR$(linenum#)
         commentloc# = INSTR(inline$, "//")
         IF commentloc# > 0 THEN 'split comments out to another field
            parm$ = LEFT$(inline$, commentloc# - 2)
            IF RIGHT$(parm$,1) = CHR$(9) OR RIGHT$(parm$,1) = CHR$(32) THEN 'check for trailing tabs
              DO
                parm$ = MID$(parm$,1,LEN(parm$) - 1) 'remove tabs
              LOOP UNTIL RIGHT$(parm$,1) <> CHR$(9) AND RIGHT$(parm$,1) <> CHR$(32)
            END IF
            StringGrid1.Cell(3,gridrow#) = parm$
            StringGrid1.Cell(4,gridrow#) = parm$
            StringGrid1.Cell(5,gridrow#) = MID$(inline$, commentloc#, LEN(inline$))
         ELSE
            StringGrid1.Cell(3,gridrow#) = inline$
            StringGrid1.Cell(4,gridrow#) = inline$
         END IF 'commentloc#
         gridrow# = gridrow# + 1
      END IF 'parmselect$
      IF FIELD$(parmnames$," ",I) = "element-count" THEN 'Find element count when ELEMENTDATA
         elementcount# = VAL(LEFT$(inline$, 1))
      END IF
      ReadNextLine
    NEXT I
END SUB 'GridParms

' This main section uses some simple mechanisms for reading information from the parbuffer
' and selecting only those components that we want to expose as field editable information
' the parsection$ refers to the main section and parsubsection$ specific WING sections.
' Parmselect$ is an array of characters using 1's and 0's to indicate which of the parms
' will be added to the StringGrid to be edited when the GridParms sub-routine is called. 
' The linenumber captured in the StringGrid is used to overlay the same line from the 
' source file with data from the parm and comment fields when saving changes.

SUB ProcessPARbuffer (parfilename$) 'Main section reading .par info from parbuffer
    openxFile(parfilename$)
    ListBox1.Clear
    InitGrid
    Label1.caption = parfilename$
    ListBox1.AddItems "Read File Name " + parfilename$
    gridrow# = 1
    linenum# = 0
    parbuffer.Position = 0
    sectioncount# = 1
    
' * * * * * * *  Check for Invalid .par file  * * * * * * *

    ReadNextLine
    IF LEFT$(inline$,1) = "0" or LEFT$(inline$,1) = "1" THEN 'check for HELI or v2.7 .par file
       DisplayWrongParFile
       GOTO PROCESSEND
    END IF
    
' * * * * * * *  MAIN processing section  * * * * * * *

    parbuffer.Position = 0
    linenum# = 0
    parsection$ = "MAIN" 'Process MAIN .par section
    subsection$ = ""
    Seek parsection$
    SkipBlankLines
    parmcount# = 7
    parmselect$ = "1000000"
    parmnames$ = "model-mass x-inertia y-inertia z-inertia inertia-product CG-position height-over-ground"
    GridParms (parmcount#, parmnames$, parmselect$)

    IF INSTR(inline$,"MOTOR") > 0 OR INSTR(inline$,"WING") > 0 THEN
       'do not read new line
    ELSE
       SkipBlankLines
    END IF
    
' * * * * * * *  MOTOR processing section  * * * * * * *    

    IF INSTR(inline$, "MOTOR") > 0 THEN
        ListBox1.AddItems "Line: " + STR$(linenum#) + " Process MOTOR section"
        parsection$ = "MOTOR" 'Process MOTOR .par section
        sectioncount# = 1
        SkipBlankLines
MOTORLABEL:     
        parmcount# = 9
        parmselect$ = "001111101"
        parmnames$ = "motor-position motor-thrust-dir prop-diam prop-pitch prop-blades prop-chord prop-rotate prop-moment motor-channel"
        GridParms (parmcount#, parmnames$, parmselect$)
        IF INSTR(inline$, "ELECTRO") = 1 THEN
           ListBox1.AddItems "Line: " + STR$(linenum#) + " MOTOR is ELECTRO"
           ReadNextLine
           parmcount# = 9
           parmselect$ = "111111110"
           parmnames$ = "voltage idling-speed idle-current idle-block-current tran-ratio tran-efficiency batt-cells batt-mAh batt-mOhm"
           GridParms (parmcount#, parmnames$, parmselect$)
        ELSEIF INSTR(inline$, "COMBUSTION") = 1 THEN
           ListBox1.AddItems "Line: " + STR$(linenum#) + " MOTOR is COMBUSTION"
           ReadNextLine
           parmcount# = 5
           parmselect$ = "10000"
           parmnames$ = "motor-size fuel-consumption friction-constant1 friction-constant2 tank-size" 
           GridParms (parmcount#, parmnames$, parmselect$)
        END IF 'ELECTRO or COMBUSTION sections

        IF INSTR(inline$,"MOTOR") > 0 OR INSTR(inline$,"WING") > 0 THEN
           'do not read new line
        ELSE
           SkipBlankLines
        END IF
    
        IF INSTR(inline$, "MOTOR") > 0 THEN 'Second and subsequent MOTOR parms
            SkipBlankLines
            sectioncount# = sectioncount# + 1
            GOTO MOTORLABEL
        ELSEIF INSTR(inline$, "WING") = 0 THEN 'check for NOT WING and NOT MOTOR
            sectioncount# = sectioncount# + 1
            GOTO MOTORLABEL
        END IF '2nd & more MOTORs
    END IF 'MOTOR section, skipped if unpowered plane or glider    

' * * * * * * *  WING processing section  * * * * * * *

    IF INSTR(inline$, "WING") = 1 THEN 'Process WING .par section
        ListBox1.AddItems "Line: " + STR$(linenum#) + " Process WING section"
        parsection$ = "WING"
        sectioncount# = 1
        subsection$ = "-left"
    END IF

    SkipBlankLines
WINGLABEL: 
    parmcount# = 6
    parmselect$ = "000000"
    parmnames$ = "wing-position wing-eX-vector wing-eY-vector polar-number span element-count"
    GridParms (parmcount#, parmnames$, parmselect$)        
    IF INSTR(inline$, "AUTOELEMENT") > 0 THEN
       ListBox1.AddItems "Line: " + STR$(linenum#) + " " + parsection$ + STR$(sectioncount#) + " is AUTOELEMENT"
       SkipBlankLines
       parmcount# = 8
              parmselect$ = "00010011"
       parmnames$ = "wing-attack-angle wing-chord-1st wing-chord-last flap-chord flap-start flap-end flap-degrees flap-channel"
       GridParms (parmcount#, parmnames$, parmselect$)  
    ELSEIF INSTR(inline$, "ELEMENTDATA") > 0 THEN
       ListBox1.AddItems "Line: " + STR$(linenum#) + " " + parsection$ + STR$(sectioncount#) + " is ELEMENTDATA Count = " + STR$(elementcount#)
       FOR I = 1 to elementcount#
         IF I = 1 THEN
            parmcount# = 5
            parmselect$ = "00011"
            parmnames$ = "elem-chord elem-attack-angle elem-flap-chord elem-flap-degrees elem-flap-channel"
         ELSE
            parmcount# = 6
            parmselect$ = "000011"
            parmnames$ = "elem-offset-pos elem-chord elem-attack-angle elem-flap-chord elem-flap-degrees elem-flap-channel"
         END IF 'First ELEMENTDATA
         GridParms (parmcount#, parmnames$, parmselect$)
       NEXT I 'elementcount#
    END IF 'AUTOELEMENT and ELEMENTDATA
    
    IF INSTR(inline$,"DRAG") > 0 OR INSTR(inline$,"POLAR") > 0 THEN
       'do not read new line
    ELSE
       SkipBlankLines
    END IF
    
    IF INSTR(inline$, "DRAG") > 0 OR INSTR(inline$, "POLAR") > 0 THEN 'second and subsequent WING sections
       ListBox1.AddItems "Line: " + STR$(linenum#) + " Exiting WING section"
    ELSE
       sectioncount# = sectioncount# + 1
       SELECT CASE sectioncount#
         CASE 2
           subsection$ = "-right"
         CASE 3   
           subsection$ = "-elevator"
         CASE 4
           subsection$ = "-rudder"
         CASE 5  
           subsection$ = "-fuselage"
         CASE 6
           subsection$ = "-right2"
         CASE 7
           subsection$ = "-left2"
       END SELECT
       GOTO WINGLABEL
    END IF

' * * * * * * *  DRAG processing section  * * * * * * *
    
    IF INSTR(inline$, "DRAG") > 0 THEN
       parsection$ = "DRAG"
       subsection$ = ""
       ListBox1.AddItems "Line: " + STR$(linenum#) + " Process DRAG section"
       sectioncount# = 1
       SkipBlankLines
       parmcount# = 4
       parmselect$ = "0111"
       parmnames$ = "drag-position drag-cross-section drag-coefficient drag-channel"
       GridParms (parmcount#, parmnames$, parmselect$)  
    END IF
      
  ListBox1.AddItems = "Completed Reading " + parfilename$

PROCESSEND:
  
END SUB 'ProcessPARbuffer

SUB OpenPARfile (openparfilename$)
    parfile.OPEN (openparfilename$, fmOpenRead)
    parbuffer.Position = 0
    parbuffer.CopyFrom (parfile, parfile.SIZE)
    parfile.CLOSE
    ProcessPARbuffer openparfilename$
END SUB 'OpenPARfile

SUB SavePARfile (saveparfilename$) 

  parfile.OPEN (saveparfilename$, fmCreate)
  ListBox1.Clear
  ListBox1.AddItems "Saving par file " + saveparfilename$
  parbuffer.Position = 0
  linenum# = 1
  gridrow# = 1
  
  FOR I = 1 to parbuffer.LineCount
    IF gridrow# <= StringGrid1.RowCount - 1 THEN 'check to see if this line is replaced
       seekline# = VAL(StringGrid1.Cell(2,gridrow#)) 'get changed line number
    ELSE
       seekline# = 99999 'if the last changed line number is encountered, set to high number
    END IF
    IF linenum# < seekline# THEN
      inline$ = parbuffer.READLINE() 'read old info 
      parfile.WRITELINE(inline$) 'write old info
      linenum# = linenum# + 1
    ELSE
      oldline$ = parbuffer.READLINE() 'read old info
      inline$ = StringGrid1.Cell(4,gridrow#) + CHR$(9) + StringGrid1.Cell(5,gridrow#) 'apply changes
      ListBox1.AddItems STR$(linenum#) + " New: " + StringGrid1.Cell(4,gridrow#) + " Old: " + oldline$
      parfile.WRITELINE(inline$) 'write new info
      linenum# = linenum# + 1
      gridrow# = gridrow# + 1
    END IF
  NEXT I

  ListBox1.AddItems "Completed Writing .par file " + saveparfilename$
  parfile.CLOSE

  'Create backup .bak file
  parfile.OPEN (saveparfilename$ + ".bak", fmCreate)
  parbuffer.Position = 0
  FOR I = 1 to parbuffer.LineCount
    inline$ = parbuffer.READLINE() 'read old info 
    parfile.WRITELINE(inline$)    'write old info  
  NEXT I
  ListBox1.AddItems "Completed Writing .bak file " + saveparfilename$ + ".bak"
  parfile.CLOSE
  
END SUB 'SavePARfile
 
'Dialog Subroutines

SUB OpenPARdialog
  OpenDialog1.Filter = "PAR files|*.par"
  OpenDialog1.InitialDir = DIR$("*.*",faDirectory)
  IF OpenDialog1.Execute THEN
     OpenPARfile (OpenDialog1.FileName)
  END IF
END SUB 'OpenPARdialog

SUB SavePARdialog
  SaveDialog1.Filter = "PAR files|*.par"
  SaveDialog1.InitialDir = DIR$("*.*",faDirectory)
  IF SaveDialog1.Execute THEN
     SavePARfile (SaveDialog1.FileName)
  END IF
END SUB 'SavePARdialog

MainForm.showmodal
